from action_hero.net import (
    IPIsValidIPAddressAction,
    IPIsValidIPv4AddressAction,
    IPIsValidIPv6AddressAction,
    URLWithHTTPResponseStatusCodeAction,
    URLIsNotReachableAction,
    URLIsReachableAction,
)

from action_hero.utils import (
    ActionHeroTestCase,
    run_only_when_when_internet_is_up,
)


class TestIPIsValidIPv4AddressAction(ActionHeroTestCase):
    def test_on_valid_ipv4_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPv4AddressAction)
        # Parse without raising any errors
        self.parser.parse_args(["--ip", "192.168.0.2"])

    def test_on_invalid_ipv4_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPv4AddressAction)
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", "500.168.0.1"])

    def test_on_valid_ipv4_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPv4AddressAction
        )
        ips = ["192.168.0.2"]
        # Parse without raising any errors
        self.parser.parse_args(["--ip", *ips])

    def test_on_invalid_ipv4_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPv4AddressAction
        )
        ips = ["500.168.0.1", "2001:db8:0:1"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *ips])

    def test_on_mixed_valid_and_invalid_ipv4_addresses_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPv4AddressAction
        )
        valid = ["192.168.0.2"]
        invalid = ["500.168.0.1"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *valid, *invalid])


class TestIPIsValidIPV6Action(ActionHeroTestCase):
    def test_on_valid_ipv6_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPv6AddressAction)
        # Parse without raising any errors
        self.parser.parse_args(["--ip", "FE80::0202:B3FF:FE1E:8329"])

    def test_on_invalid_ipv6_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPv6AddressAction)
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", "10.168.0.1"])

    def test_on_valid_ipv6_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPv6AddressAction
        )
        ips = ["2001:db8:1::ab9:C0A8:102", "FE80::0202:B3FF:FE1E:8329"]
        # Parse without raising any errors
        self.parser.parse_args(["--ip", *ips])

    def test_on_invalid_ipv6_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPv6AddressAction
        )
        ips = ["122.168.0.1", "0.0.0.0"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *ips])

    def test_on_mixed_valid_and_invalid_ipv6_addresses_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPv6AddressAction
        )
        valid = ["192.168.0.2"]
        invalid = ["122.168.0.1", "0.0.0.0"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *valid, *invalid])


class TestIPIsValidIPAddressAction(ActionHeroTestCase):
    def test_on_valid_ipv4_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPAddressAction)
        # Parse without raising any errors
        self.parser.parse_args(["--ip", "192.168.0.2"])

    def test_on_invalid_ipv4_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPAddressAction)
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", "500.168.0.1"])

    def test_on_valid_ipv4_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        ips = ["192.168.0.2"]
        # Parse without raising any errors
        self.parser.parse_args(["--ip", *ips])

    def test_on_invalid_ipv4_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        ips = ["500.168.0.1", "2001:db8:0:1"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *ips])

    def test_on_mixed_valid_and_invalid_ipv4_addresses_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        valid = ["192.168.0.2"]
        invalid = ["500.168.0.1"]
        # Asser error raised on parse
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *valid, *invalid])

    def test_on_valid_ipv6_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPAddressAction)
        # Parse without raising any errors
        self.parser.parse_args(["--ip", "FE80::0202:B3FF:FE1E:8329"])

    def test_on_invalid_ipv6_address(self):
        self.parser.add_argument("--ip", action=IPIsValidIPAddressAction)
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", "1000.168.0.1"])

    def test_on_valid_ipv6_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        ips = ["2001:db8:1::ab9:C0A8:102", "FE80::0202:B3FF:FE1E:8329"]
        # Parse without raising any errors
        self.parser.parse_args(["--ip", *ips])

    def test_on_invalid_ipv6_address_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        ips = [":AA:2001:db8:1::ab9:C0A8:102", ":::FE80::02:B3:FE1E:8329"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *ips])

    def test_on_mixed_valid_and_invalid_ipv4_and_ipv6_addresses_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        valid = ["192.168.0.2"]
        invalid = ["x122.168.0.1", "0.0.0.0.0.0"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *valid, *invalid])

    def test_on_mixed_valid_ipv4_and_ipv6_addresses_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        ips = [
            # "FE80::0202:B3FF:FE1E:8329",
            "192.168.0.2",
            "20.0.0.120",
        ]
        # Parse without raising any errors
        self.parser.parse_args(["--ip", *ips])

    def test_on_mixed_invalid_ipv4_and_ipv6_addresses_list(self):
        self.parser.add_argument(
            "--ip", nargs="+", action=IPIsValidIPAddressAction
        )
        invalid = [
            "a.168.0.2",
            "120",
            ":AA:2001:db8:1::ab9:C0A8:102",
            ":::FE80::02:B3:FE1E:8329",
        ]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--ip", *invalid])


class TestURLIsReachableAction(ActionHeroTestCase):
    @run_only_when_when_internet_is_up(urls=["http://www.google.com"])
    def test_on_reachable_url(self):
        self.parser.add_argument("--url", action=URLIsReachableAction)
        url = "http://google.com"
        self.parser.parse_args(["--url", url])

    @run_only_when_when_internet_is_up(
        urls=["http://www.google.com", "http://www.microsoft.com"]
    )
    def test_on_reachable_urls(self):
        self.parser.add_argument(
            "--url", nargs="+", action=URLIsReachableAction
        )
        urls = ["http://www.google.com", "http://www.microsoft.com"]
        self.parser.parse_args(["--url", *urls])

    @run_only_when_when_internet_is_up(urls=["http://www.google.com"])
    def test_on_unreachable_url(self):
        self.parser.add_argument("--url", action=URLIsReachableAction)
        unreachable = "AAA"
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--url", unreachable])

    @run_only_when_when_internet_is_up(urls=["http://www.google.com"])
    def test_on_unreachable_urls(self):
        self.parser.add_argument(
            "--url", nargs="+", action=URLIsReachableAction
        )
        unreachable = ["AAA", "httt://www.notreal.com"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--url", *unreachable])


class TestURLIsNotReachableAction(ActionHeroTestCase):
    @run_only_when_when_internet_is_up(urls=["http://www.google.com"])
    def test_on_reachable_url(self):
        self.parser.add_argument("--url", action=URLIsNotReachableAction)
        url = "http://google.com"
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--url", url])

    @run_only_when_when_internet_is_up(
        urls=["http://www.google.com", "http://www.microsoft.com"]
    )
    def test_on_reachable_urls(self):
        self.parser.add_argument(
            "--url", nargs="+", action=URLIsNotReachableAction
        )
        urls = ["http://www.google.com", "http://www.microsoft.com"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--url", *urls])

    @run_only_when_when_internet_is_up(urls=["http://www.google.com"])
    def test_on_unreachable_url(self):
        self.parser.add_argument("--url", action=URLIsNotReachableAction)
        unreachable = "AAA"
        self.parser.parse_args(["--url", unreachable])

    @run_only_when_when_internet_is_up(urls=["http://www.google.com"])
    def test_on_unreachable_urls(self):
        self.parser.add_argument(
            "--url", nargs="+", action=URLIsNotReachableAction
        )
        unreachable = ["AAA", "YYY"]
        self.parser.parse_args(["--url", *unreachable])


class TestURLWithHTTPResponseStatusCodeAction(ActionHeroTestCase):
    @run_only_when_when_internet_is_up(urls=["http://www.google.com"])
    def test_on_reachable_url(self):
        self.parser.add_argument(
            "--url",
            action=URLWithHTTPResponseStatusCodeAction,
            # Request success codes 2xx
            action_values=["200", "201", "202", "203", "204"],
        )
        url = "http://google.com"
        self.parser.parse_args(["--url", url])

    def test_on_unreachable_url(self):
        self.parser.add_argument(
            "--url",
            action=URLWithHTTPResponseStatusCodeAction,
            # Request failure codes 4xx, 5xx
            action_values=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "500",
                "501",
                "502",
                "503",
            ],
        )
        unreachable = "AAA"
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--url", unreachable])



import unittest
import pathlib
import os

from action_hero.getters import (
    get_about,
    get_config_as_dict,
    get_readme_contents,
    get_readme_content_type,
)


class TestGetAbout(unittest.TestCase):
    def setUp(self):
        self.about = get_about()

    def test_get_about_returns_dict(self):
        self.assertIsInstance(self.about, dict)

    def test_get_about_returns_nonempty(self):
        self.assertGreater(len(self.about), 0)


class TestGetConfigAsGet(unittest.TestCase):
    def test_return_value_as_a_dict(self):
        import meta

        self.assertIsInstance(get_config_as_dict(meta, "about.ini"), dict)


class TestGetReadme(unittest.TestCase):
    def setUp(self):
        self.about = get_about()

    def test_about_has_readme_key(self):
        self.assertIn("readme_filename", self.about["PROJECT"])

    def test_readme_exists(self):
        path = self.about["PROJECT"]["readme_filename"]
        self.assertTrue(os.path.isfile(path))

    def test_readme_is_of_known_readme_type(self):
        path = self.about["PROJECT"]["readme_filename"]
        suffix = pathlib.Path(path).suffix[1:]
        self.assertIn(suffix, ["md", "rst"])

    def test_readme_is_of_known_content_type(self):
        self.assertIn(
            get_readme_content_type(),
            ["text/markdown", "text/x-rst", "text/plain"],
        )

    def test_readme_is_nonempty(self):
        self.assertNotEqual(get_readme_contents(), "")



from action_hero.utils import CheckAction

from action_hero.types_utils import (
    is_convertible_to_int,
    is_convertible_to_float,
    is_convertible_to_UUID,
    is_truthy,
)


__all__ = [
    "IsConvertibleToFloatAction",
    "IsConvertibleToIntAction",
    "IsConvertibleToUUIDAction",
    "IsFalsyAction",
    "IsTruthyAction",
]


class IsConvertibleToIntAction(CheckAction):
    """Check if value is convertible to int"""

    func = is_convertible_to_int
    err_msg_singular = "Value is not convertible to int"
    err_msg_plural = "Atleast one value is not convertible to int"


class IsConvertibleToFloatAction(CheckAction):
    """Check if value is convertible to float"""

    func = is_convertible_to_float
    err_msg_singular = "Value is not convertible to float"
    err_msg_plural = "Atleast one value is not convertible to float"


class IsTruthyAction(CheckAction):
    """Checks if value is truthy"""

    func = is_truthy
    err_msg_singular = "Value is falsy"
    err_msg_plural = "Atleast one value is falsy"


class IsFalsyAction(CheckAction):
    """Checks if value is falsy"""

    def func(value):
        return not is_truthy(value)

    err_msg_singular = "value is falsy"
    err_msg_plural = "Atleast one value is falsy"


class IsConvertibleToUUIDAction(CheckAction):
    """Checks if value is convertible to UUID"""

    func = is_convertible_to_UUID
    err_msg_singular = "Value cannot be converted to UUID"
    err_msg_plural = "Atleast one value cannot be converted to UUID"



from setuptools import find_packages, setup

from action_hero.getters import (
    get_about,
    get_readme_contents,
    get_readme_content_type,
)


about = get_about()
long_description = get_readme_contents()
long_description_content_type = get_readme_content_type()


setup(
    name=about["PROJECT"]["program_name"],
    version=about["PROJECT"]["version"],
    description=about["PROJECT"]["description"],
    long_description=long_description,
    long_description_content_type=long_description_content_type,
    author=about["AUTHOR"]["name"],
    author_email=about["AUTHOR"]["email"],
    python_requires=">={}".format(about["PROJECT"]["requires_python"]),
    url=about["PROJECT"]["url"],
    license=about["PROJECT"]["license"],
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=["importlib_resources", "requests"],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
        "Topic :: Terminals",
        'Natural Language :: English',
    ],
)



import os
import stat
import pathlib


__all__ = [
    "add_execute_permission",
    "create_directory",
    "create_file",
    "get_extension",
    "is_empty_file",
    "is_executable_directory",
    "is_executable_file",
    "is_executable_path",
    "is_existing_directory",
    "is_existing_file",
    "is_existing_or_creatable_path",
    "is_existing_path",
    "is_readable_directory",
    "is_readable_file",
    "is_readable_path",
    "is_symbolic_link",
    "is_valid_directory",
    "is_valid_file",
    "is_valid_path",
    "is_writable_directory",
    "is_writable_file",
    "is_writable_path",
    "remove_execute_permission",
    "remove_read_permission",
    "remove_write_permission",
    "resolve_path",
]


def get_extension(path):
    """Get file extension/prefix

    The dot is strippped if there is one.

    Args:
        path (str): Filename

    Returns:
        (str) File extension or blank string

    """
    suffix = pathlib.Path(path).suffix
    # Return suffix is nonempty without the starting dot
    # or blank string if theres none
    if suffix and suffix.startswith("."):
        return suffix[1:]
    else:
        return ""


def create_file(path):
    """Creates a file with name of path

    When exist_ok=True is set to True on touch(), the file's modification date
    will be set yet a new file is not created over it. Else, when a file
    already exists there, it might raise a FileExistsError. TODO: Visit this
    behavior again.

    Args:
        path (str): Path to create file
    """
    pathlib.Path(path).touch(exist_ok=True)


def create_directory(path):
    """Creates a directory with name of path

    Args:
        path (str): Path to create directory

    Raises:
        FileExistsError: Raised if path is already present
    """
    os.mkdir(path)


def is_symbolic_link(path):
    """Return True if path is existing directory that is a symbolic link"""
    return os.path.islink(path)


def is_existing_directory(path):
    """Returns True if path is an existing directory"""
    return os.path.isdir(path)


def is_readable_directory(path):
    """Returns True if path is a directory with read permissible flag set"""
    return is_existing_directory(path) and is_readable_path(path)


def is_writable_directory(path):
    """Returns True if path is a directory with write permissible flag set"""
    return is_existing_directory(path) and is_writable_path(path)


def is_executable_directory(path):
    """Returns True if path is a directory with execute permissible flag set"""
    return is_existing_directory(path) and is_executable_path(path)


def is_existing_file(path):
    """Returns True if path is an existing file"""
    return os.path.isfile(path)


def is_readable_file(path):
    """Returns True if path is a file with read permissible flag set"""
    return is_existing_file(path) and is_readable_path(path)


def is_writable_file(path):
    """Returns True if path is a file with write permissible flag set"""
    return is_existing_file(path) and is_writable_path(path)


def is_executable_file(path):
    """Return True if path is a file with execute permissible flag set"""
    return is_existing_file(path) and is_executable_path(path)


def is_existing_path(path):
    """Returns True if path exists"""
    return os.path.exists(path)


def is_writable_path(path):
    """Returns True if path has write permissible flag set"""
    return os.access(path, os.W_OK)


def is_readable_path(path):
    """Returns True if path has read permission flag set"""
    return os.access(path, os.R_OK)


def is_executable_path(path):
    """Returns True if path has execute permission flag set"""
    return os.access(path, os.X_OK)


def resolve_path(path):
    """Returns resolved canonical path removing symbolic links if present"""
    return os.path.realpath(path)
    # return str(pathlib.Path(path).resolve())


def is_existing_or_creatable_path(path):
    """Returns True if path already exists or is creatable by current User

    This solution is a work-around, in that, an ignorable tempfile with a
    shortlife is created and removed.
    Source: https://stackoverflow.com/a/48499049/225903

    For a better, although far more complicated solution, read:
    https://stackoverflow.com/a/34102855/225903

    Args:
        path (str): Path to check for existence or creatability

    """
    try:
        # Mode x: Open for exclusive creation, fail if the path already exists
        with open(path, mode="x") as _:
            # Raises OSError if:
            # Condition 1. file exists
            # Condition 2. Unable to create at path

            # Else successfully able to create at path
            return True

    except OSError:
        # Handle Condition 1. Path exists
        if os.path.isfile(path) or os.path.isdir(path):
            return True
        # Handle Condition 2. Unable to create at path
        return False


def is_valid_path(path):
    """Returns True if path already exists or is creatable by current User"""
    return is_existing_or_creatable_path(path)


def is_valid_directory(path):
    """ Returns True if directoy as path already exists or is creatable by
    current User.

    """
    return is_valid_path(path)


def is_valid_file(path):
    """Returns True if file as path already exists or is creatable by current
    User

    """
    return is_valid_path(path)


def remove_write_permission(path):
    """Remove write permissions and keep other permissions intact.

    Source:
        https://stackoverflow.com/a/38511116/225903

    Args:
        path (str):  The path whose permissions to alter.

    """
    NO_USER_WRITING = ~stat.S_IWUSR
    NO_GROUP_WRITING = ~stat.S_IWGRP
    NO_OTHER_WRITING = ~stat.S_IWOTH
    NO_WRITING = NO_USER_WRITING & NO_GROUP_WRITING & NO_OTHER_WRITING

    current_permissions = stat.S_IMODE(os.lstat(path).st_mode)
    os.chmod(path, current_permissions & NO_WRITING)


def remove_read_permission(path):
    """Remove read permissions and keep other permissions intact.

    Source:
        https://stackoverflow.com/a/38511116/225903

    Args:
        path (str):  The path whose permissions to alter.
    """
    NO_USER_READING = ~stat.S_IRUSR
    NO_GROUP_READING = ~stat.S_IRGRP
    NO_OTHER_READING = ~stat.S_IROTH
    NO_READING = NO_USER_READING & NO_GROUP_READING & NO_OTHER_READING

    current_permissions = stat.S_IMODE(os.lstat(path).st_mode)
    os.chmod(path, current_permissions & NO_READING)


def remove_execute_permission(path):
    """Remove execute permissions and keep other permissions intact.

    Source:
        https://stackoverflow.com/a/38511116/225903

    Args:
        path (str):  The path whose permissions to alter.

    """
    NO_USER_EXECUTING = ~stat.S_IXUSR
    NO_GROUP_EXECUTING = ~stat.S_IXGRP
    NO_OTHER_EXECUTING = ~stat.S_IXOTH
    NO_EXECUTING = (
        NO_USER_EXECUTING & NO_GROUP_EXECUTING & NO_OTHER_EXECUTING
    )

    current_permissions = stat.S_IMODE(os.lstat(path).st_mode)
    os.chmod(path, current_permissions & NO_EXECUTING)


def add_execute_permission(path):
    """Add execute permission and keep other permissions intact.

    Source:
        https://stackoverflow.com/a/38511116/225903

    Args:
        path:  The path whose permissions to alter.

    """
    USER_EXECUTING = stat.S_IXUSR
    GROUP_EXECUTING = stat.S_IXGRP
    OTHER_EXECUTING = stat.S_IXOTH
    EXECUTING = USER_EXECUTING | GROUP_EXECUTING | OTHER_EXECUTING

    current_permissions = stat.S_IMODE(os.lstat(path).st_mode)
    os.chmod(path, current_permissions | EXECUTING)


def is_empty_file(path):
    """Returns True if file is empty"""
    return is_existing_file(path) and os.stat(path).st_size == 0
